/**************************** DISPLAY *****************************/

// Controls the display in normal clock mode. 

void control_display(void)
{
   //Tick-Tack
   if (effect_ctrl.ticktack && !((ctrl_struct.alarming && !snoozetimer) || ctrl_struct.countdown_alarming)) //Tick every time only when no alarm is active
	tick();
   else if (ctrl_struct.countdown_is_set && !(ctrl_struct.alarming && !snoozetimer)) //Tick in countdown-mode only when no alarm is active
	tick();

   if (ctrl_struct.timeunknown && (time.s % 2)) {
      	display_str("        ", TRUE);
      	effect_ctrl.time = TRUE; // Show floating effect when time is unknown, its nice!!!!

   } else {
	if (ctrl_struct.alarming && (time.s % 2)) {
	 	display_str(LANG_ALERT, TRUE); // Show floating effect when alarm is active, its nice!!!!
		effect_ctrl.time = TRUE;
	} else if (ctrl_struct.countdown_alarming && (time.s % 2)) {
		display_str(LANG_TIMER, TRUE); // Show floating effect when timer is active, its nice!!!!
		effect_ctrl.time = TRUE;

	} else {
	  	if (time.s < 50)
			{
      	     		display_time(time.h, time.m, time.s, effect_ctrl.time);
			}

	  	else if (time.s < 55) {
			if (ctrl_struct.countdown_is_set)
				{
				display_countdown(countdown, effect_ctrl.message_timer);
				}
			else
				//display_number(pwm_sequence_speed);
	     			display_str(MESSAGE, effect_ctrl.message_timer);

	  	} else { 
             		display_date(time.d, time.mo, time.y, DATE, effect_ctrl.date);
			}
		}
   	}

  if (ctrl_struct.alarm_on || ctrl_struct.countdown_is_set)
	display[0] |= 0x1;
  else 
	display[0] &= ~0x1;
}

// We can display the current date!

void display_date(uint8_t d, uint8_t m, uint8_t y, uint8_t style, uint8_t float_in) {

  uint8_t i;
  // no shadow
  effect_ctrl.create_shadow_en = FALSE;

  // This type is mm-dd-yy OR dd-mm-yy depending on our pref.
  if (style == DATE) {
    display3[0] = 0;
    display3[3] = display3[6] = 0x01;     // put points between num

    if (time.r == REGION_US) {
      // mm-dd-yy
      display3[1] = pgm_read_byte(numbertable_p + (m / 10));
      display3[2] = pgm_read_byte(numbertable_p + (m % 10));
      display3[4] = pgm_read_byte(numbertable_p + (d / 10));
      display3[5] = pgm_read_byte(numbertable_p + (d % 10));
    } else {
      // dd-mm-yy
      display3[1] = pgm_read_byte(numbertable_p + (d / 10));
      display3[2] = pgm_read_byte(numbertable_p + (d % 10));
      display3[4] = pgm_read_byte(numbertable_p + (m / 10));
      display3[5] = pgm_read_byte(numbertable_p + (m % 10));
    }
    // the yy part is the same
    display3[7] = pgm_read_byte(numbertable_p + (y / 10));
    display3[8] = pgm_read_byte(numbertable_p + (y % 10));

    //this does the float-in-effect if enabled
    if (float_in == TRUE && effect_ctrl.effect == TRUE)
	{
	effect_ctrl.date = FALSE;
	float_display();
 	} 
    else 
	{
	for (i=0;i<9;i++)
		display[i]=display3[i];
	}			

  } else if (style == DAY) {
    // This is more "Sunday June 21" style

    // Display the day first
    display[8] = display[7] = 0;
    switch (calc_dotw()) {
    case 0:
      display_str(LANG_SUNDAY, FALSE); break;
    case 1:
      display_str(LANG_MONDAY, FALSE); break;
    case 2:
      display_str(LANG_TUESDAY, FALSE); break;
    case 3:
      display_str(LANG_WEDNESDAY, FALSE); break;
    case 4:
      display_str(LANG_THURSDAY, FALSE); break;
    case 5:
      display_str(LANG_FRIDAY, FALSE); break;
    case 6:
      display_str(LANG_SATURDAY, FALSE); break;
    }
    
    // wait one seconds about
    delayms(1000);

    // Then display the month and date
    display[6] = display[5] = display[4] = 0;
    switch (m) {
    case 1:
      display_str(LANG_JAN, FALSE); break;
    case 2:
      display_str(LANG_FEB, FALSE); break;
    case 3:
      display_str(LANG_MAR, FALSE); break;
    case 4:
      display_str(LANG_APR, FALSE); break;
    case 5:
      display_str(LANG_MAY, FALSE); break;
    case 6:
      display_str(LANG_JUN, FALSE); break;
    case 7:
      display_str(LANG_JUL, FALSE); break;
    case 8:
      display_str(LANG_AUG, FALSE); break;
    case 9:
      display_str(LANG_SEP, FALSE); break;
    case 10:
      display_str(LANG_OCT, FALSE); break;
    case 11:
      display_str(LANG_NOV, FALSE); break;
    case 12:
      display_str(LANG_DEZ, FALSE); break;
    }
    display[7] = pgm_read_byte(numbertable_p + (d / 10));
    display[8] = pgm_read_byte(numbertable_p + (d % 10));

    
  }
}

// This displays a time on the clock

void display_time(uint8_t h, uint8_t m, uint8_t s, uint8_t float_in) {
  
  uint8_t i;

  // seconds and minutes are at the end

   // This is for saving "last display" for over-blending in time-mode
   if (float_in == FALSE && displaymode == SHOW_TIME && effect_ctrl.effect == TRUE)
	{
   	effect_ctrl.create_shadow_en = TRUE;
   	create_shadow();
	}
   else
	effect_ctrl.create_shadow_en = FALSE;

  display3[8] =  pgm_read_byte(numbertable_p + (s % 10));
  display3[7] =  pgm_read_byte(numbertable_p + (s / 10));
  display3[6] = 0;
  display3[5] =  pgm_read_byte(numbertable_p + (m % 10));
  display3[4] =  pgm_read_byte(numbertable_p + (m / 10)); 
  display3[3] = 0;

  // check euro (24h) or US (12h) style time
  if (time.r == REGION_US) {
    display3[2] =  pgm_read_byte(numbertable_p + ( (((h+11)%12)+1) % 10));
    display3[1] =  pgm_read_byte(numbertable_p + ( (((h+11)%12)+1) / 10));

    // We use the '*' as an am/pm notice
    if (h >= 12)
      display3[0] |= 0x2;  // 'pm' notice
    else 
      display3[0] &= ~0x2;  // 'pm' notice
  } else {
    display3[2] =  pgm_read_byte(numbertable_p + ( (h%24) % 10));
    display3[1] =  pgm_read_byte(numbertable_p + ( (h%24) / 10));
  }

    //this does the float-in-effect if enabled
    if (float_in == TRUE && effect_ctrl.effect == TRUE)
	{
	effect_ctrl.time = FALSE;
	float_display();
 	} 
    else 
	{
	for (i=0;i<9;i++)
		display[i]=display3[i];
	}

}

// Kinda like display_time but just hours and minutes

void display_alarm(uint8_t h, uint8_t m){

  // no shadow
  effect_ctrl.create_shadow_en = FALSE;
 
  display[8] = 0;
  display[7] = 0;
  display[6] = 0;
  display[5] = pgm_read_byte(numbertable_p + (m % 10));
  display[4] = pgm_read_byte(numbertable_p + (m / 10)); 
  display[3] = 0;

  // check euro or US style time
  if (time.r == REGION_US) {
    	if (h >= 12) {
		display[0] |= 0x2;  // 'pm' notice
      		display[7] = pgm_read_byte(alphatable_p + 'p' - 'a');
    	} else {
      		display[7] = pgm_read_byte(alphatable_p + 'a' - 'a');
      		display[0] &= ~0x2;  // 'am' notice
    	}
    	display[8] = pgm_read_byte(alphatable_p + 'm' - 'a');

    	display[2] =  pgm_read_byte(numbertable_p + ( (((h+11)%12)+1) % 10));
    	display[1] =  pgm_read_byte(numbertable_p + ( (((h+11)%12)+1) / 10));
  } else {
      	display[2] =  pgm_read_byte(numbertable_p + ( (((h+23)%24)+1) % 10));
    	display[1] =  pgm_read_byte(numbertable_p + ( (((h+23)%24)+1) / 10));
	}

}

//display the countdown timer

void display_countdown(uint8_t c, uint8_t float_in)
{

  uint8_t i;

  // no shadow
  effect_ctrl.create_shadow_en = FALSE;

  display3[1] = pgm_read_byte(alphatable_p + 't' - 'a');
  display3[2] = pgm_read_byte(alphatable_p + 'i' - 'a');
  display3[3] = pgm_read_byte(alphatable_p + 'm' - 'a');
  display3[4] = pgm_read_byte(alphatable_p + 'e' - 'a');
  display3[5] = pgm_read_byte(alphatable_p + 'r' - 'a');
  display3[6] = pgm_read_byte(numbertable_p + (c / 100));
  display3[7] = pgm_read_byte(numbertable_p + ((c % 100) / 10));
  display3[8] = pgm_read_byte(numbertable_p + (c % 10));

  //this does the float-in-effect if enabled
  if (float_in == TRUE && effect_ctrl.effect == TRUE)
	{
	effect_ctrl.message_timer = FALSE; //disable effect next time
	float_display();
 	} 
  else 
	{
	for (i=0;i<9;i++)
		display[i]=display3[i]; //set only new segment, from left to right
	}

}

// display words (menus, prompts, etc)

void display_str(char *s, uint8_t float_in) {

  uint8_t i,j;

  // no shadow
  effect_ctrl.create_shadow_en = FALSE;

  // don't use the lefthand dot/slash digit
  display3[0] = 0;

  // up to 8 characters
  for (i=1; i<9; i++) {
    // check for null-termination
    if (s[i-1] == 0) {
      break; 
      }

    //set new segment, from left to right
    // Numbers and leters are looked up in the font table!
    if ((s[i-1] >= 'a') && (s[i-1] <= 'z')) {
      display3[i] =  pgm_read_byte(alphatable_p + s[i-1] - 'a');
    } else if ((s[i-1] >= '0') && (s[i-1] <= '9')) {
      display3[i] =  pgm_read_byte(numbertable_p + s[i-1] - '0');
    } else {
      display3[i] = 0;      // spaces and other stuff are ignored :(
    }
  }

  //this does the float-in-effect if enabled. only for strings with 8 characters !
  if (float_in == TRUE && effect_ctrl.effect == TRUE && i==9)
	{
	effect_ctrl.message_timer = FALSE; //disable effect next time
	float_display();
 	} 
else 
	{
	for (j=0;j<i;j++)
		display[j]=display3[j]; //set only new segment, from left to right
	}
}

//Displays numbers from 0 to 65535. Normally not used but maybe interesting for debugging.

//void display_number(uint16_t number)
//{
  //no shadow
//  effect_ctrl.create_shadow_en = FALSE;
//
//  display[1] = display[2] = display[3] = 0;
//  display[4] = pgm_read_byte(numbertable_p + (number / 10000)); 
//  number = number%10000;
//  display[5] = pgm_read_byte(numbertable_p + (number / 1000));
//  number = number%1000;
//  display[6] = pgm_read_byte(numbertable_p + (number / 100)); 
//  number = number%100;
//  display[7] = pgm_read_byte(numbertable_p + (number / 10));
//  number = number%10;
//  display[8] = pgm_read_byte(numbertable_p + (number)); 
//}

//Creates the float display effect. This must be called after a new display is temporarly build in display3[];
//Otherwise it will not work!

void float_display()
{
uint8_t i,j,k;

for (i=0;i<9;i++)
	{
	for (j=9;j>i;j--) //set every old segment one to right
		{
		display[j+1]=display[j];
		display[j]=0; 
		}
	 //wait short time
	delayms(50);
	k=i;
	//set new segments, from left to right
	for (j=0;j<=i;j++)
		display[j]=display3[8-(k--)];
	}
}

//Creates the shadow display for overblending
//If its called before building a new display, it will create a shadow of the old display.
//Otherwise it will not create a shadow.

void create_shadow()
{
 uint8_t i;
 for (i=0; i<=DISPLAYSIZE; i++)
	display2[i] = display[i];
}

/************************* LOW LEVEL DISPLAY ************************/

// This changes and updates the display
// We use the digit/segment table to determine which
// pins on the MAX6921 to turn on

void setdisplay(uint8_t digit, uint8_t segments) {
  uint32_t t, d = 0;  // we only need 20 bits but 32 will do
  uint8_t i;

  // Set the digit selection pin
  d |= _BV(pgm_read_byte(digittable_p + digit));

  
  // Set the individual segments for this digit
  for (i=0; i<8; i++) {
    if (segments & _BV(i)) { //_BV(i) generates a binary number, where bit i is 1. The statement looks, which segment should light up. 
      t = 1; // Set first bit to one
      t <<= pgm_read_byte(segmenttable_p + i); //Shift the first bit (which is 1) to bit n (n is read from Segmenttable), equivalent to  t = t << .., so that the corresponding pin on the max 6921 will be enabled.
      d |= t; // Combine this with data d.
    }
  }

  // Shift the data out to the display
  vfd_send(d);
}

// send raw data to display, its pretty straightforward. Just send 32 bits via SPI
// the bottom 20 define the segments

void vfd_send(uint32_t d) {
  // send lowest 20 bits
  cli();       // to prevent flicker we turn off interrupts
  spi_xfer(d >> 16);
  spi_xfer(d >> 8);
  spi_xfer(d);

  // latch data
  VFDLOAD_PORT |= _BV(VFDLOAD);
  VFDLOAD_PORT &= ~_BV(VFDLOAD);
  sei();
}

// Send 1 byte via SPI

void spi_xfer(uint8_t c) {

  SPDR = c;
  while (! (SPSR & _BV(SPIF)));
}
 
