/*******************************Buttons********************************/

void initbuttons(void) {
    DDRB =  _BV(VFDCLK) | _BV(VFDDATA) | _BV(SPK1) | _BV(SPK2);
    DDRD = _BV(BOOST) | _BV(VFDSWITCH);
    DDRC = _BV(VFDLOAD) | _BV(VFDBLANK) | _BV(LED1) | _BV(LED2) | _BV(LED3) | _BV(LED4);
    PORTD = _BV(BUTTON1) | _BV(BUTTON3) | _BV(ALARM);
    PORTB = _BV(BUTTON2);

    //enable this and disable other line for original state
    PCICR = _BV(PCIE0) | _BV(PCIE2);
    //PCICR = _BV(PCIE0) | _BV(PCIE1) | _BV(PCIE2);

    PCMSK0 = _BV(PCINT0);

    //enable this and disable other line for original state
    PCMSK2 = _BV(PCINT21) | _BV(PCINT20);    
    //PCMSK2 = _BV(PCINT17) | _BV(PCINT21) | _BV(PCINT20);
 
}

/**************************** RTC & ALARM *****************************/

void clock_init(void) {
  // we store the time in EEPROM when switching from power modes so its
  // reasonable to start with whats in memory
  time.h = eeprom_read_byte((uint8_t *)EE_HOUR) % 24;
  time.m = eeprom_read_byte((uint8_t *)EE_MIN) % 60;
  time.s = eeprom_read_byte((uint8_t *)EE_SEC) % 60;
  time.z = eeprom_read_byte((uint8_t *)EE_ZONE);

  /*
    // if you're debugging, having the makefile set the right
    // time automatically will be very handy. Otherwise don't use this
  time.sh = TIMEHOUR;
  time.sm = TIMEMIN;
  time.ss = TIMESEC + 10;
  */

  // Set up the stored alarm time and date
  alarm_m = eeprom_read_byte((uint8_t *)EE_ALARM_MIN) % 60;
  alarm_h = eeprom_read_byte((uint8_t *)EE_ALARM_HOUR) % 24;

  time.y = eeprom_read_byte((uint8_t *)EE_YEAR) % 100;
  time.mo = eeprom_read_byte((uint8_t *)EE_MONTH) % 13;
  time.d = eeprom_read_byte((uint8_t *)EE_DAY) % 32;

  ctrl_struct.restored = 1;

  // Turn on the RTC by selecting the external 32khz crystal
  // 32.768 / 32 = (256*4) which is exactly an 8-bit timer overflow
  ASSR |= _BV(AS2); // use crystal
  TCCR2B = _BV(CS21) | _BV(CS20); // div by 32
  // We will overflow four times a second, and call an interrupt

  // enable interrupt
  TIMSK2 = _BV(TOIE2);

  // enable all interrupts!
  sei();
}

/**************************** SPEAKER *****************************/
// Set up the speaker to prepare for beeping!

void speaker_init(void) {

  // read the preferences for high/low volume
  ctrl_struct.volume = eeprom_read_byte((uint8_t *)EE_VOLUME);

  // We use the built-in fast PWM, 8 bit timer
  PORTB |= _BV(SPK1) | _BV(SPK2); 

  // Turn on PWM outputs for both pins
  TCCR1A = _BV(COM1B1) | _BV(COM1B0) | _BV(WGM11);
  if (ctrl_struct.volume) {
    TCCR1A |= _BV(COM1A1);
  } 
  TCCR1B = _BV(WGM13) | _BV(WGM12);

  // start at 4khz:  250 * 8 multiplier * 4000 = 8mhz
  ICR1 = 250;
  OCR1B = OCR1A = ICR1 / 2;
}

/**************************** BOOST *****************************/

// We control the boost converter by changing the PWM output
// pins
void boost_init(uint8_t brightness) {
  // Set PWM value, don't set it so high that
  // we could damage the MAX chip or display
  if (brightness > 100)
    brightness = 100;

  // Or so low its not visible
  if (brightness < 10)
    brightness = 10;

  if (brightness <= 10) {
	  OCR0A = 30; 
	} else if (brightness <= 20) {
	  OCR0A = 35;
	} else if (brightness <= 30) {
	  OCR0A = 40;
	} else if (brightness <= 40) {
	  OCR0A = 45;
	} else if (brightness <= 50) {
	  OCR0A = 50;
	} else if (brightness <= 60) {
	  OCR0A = 60;
	} else if (brightness <= 70) {
	  OCR0A = 70;
	} else if (brightness <= 80) {
	  OCR0A = 80;
	} else if (brightness <= 90) {
	  OCR0A = 85;
	} else if (brightness <= 100) {
	  OCR0A = 90;
	}  else {
	  OCR0A = 30;
	}

  // fast PWM, set OC0A (boost output pin) on match
  TCCR0A = _BV(WGM00) | _BV(WGM01);  

  // Use the fastest clock
  TCCR0B = _BV(CS00);
 
  TCCR0A |= _BV(COM0A1);
  TIMSK0 |= _BV(TOIE0); // turn on the interrupt for muxing
  sei();
} 

/************************* SPI(DISPLAY) **************************/

// Setup SPI

void vfd_init(void) {
  SPCR  = _BV(SPE) | _BV(MSTR) | _BV(SPR0);
}
