
/************************** Interrupt handlers ***************************/

/*******Update Display and Alarming * called @ ~30khz (31.25 khz)*********/

void update_display(void)
{
// allow other interrupts to go off while we're doing display updates
sei();

static uint8_t pwm_div;

if (effect_ctrl.light == FALSE)
	{
	pwm_div++;
	if (pwm_div == 3)
		{
		pwm_div = 0;
		ambilight_pwm();
		}
	}

// kick the dog
kickthedog();

//use ambilight pwm if light is switched to "puls" in menu
//its called via devider,otherwise its much for this little mcu ;(

// divide down to 100Hz * digits
muxdiv++;
if (muxdiv < MUX_DIVIDER)
    return;
muxdiv = 0;

// now at 100Hz * digits
if (milliseconds < 65534)
	milliseconds++;

// Cycle through each digit in the display
if (currdigit >= DISPLAYSIZE)
	currdigit = 0;

// This creates a shadow of the last display for overblending in time-mode 
if (overblend && effect_ctrl.create_shadow_en == TRUE && effect_ctrl.effect == TRUE)
	{
	overblend--;
	setdisplay(currdigit, display2[currdigit]);
	}

// Set the current display's segments
setdisplay(currdigit, display[currdigit]);

// and go to the next
currdigit++;

// check if we should have the alarm on
if ((ctrl_struct.alarming && !snoozetimer) || ctrl_struct.countdown_alarming) {
	alarmdiv++;
    	if (alarmdiv > ALARM_DIVIDER) {
      		alarmdiv = 0;
    	} else {
      		return;
    	}
	// This part only gets reached at 1Hz

	// This sets the buzzer frequency
	if (ctrl_struct.alarming)
		ICR1 = 250;
	else 
		ICR1 = 150;
	OCR1A = OCR1B = ICR1/2;

	// ok alarm is ringing!
	if (effect_ctrl.beeping & 0x01) { // first bit indicates pulsing alarm state
		effect_ctrl.beeping &= ~0x01;
      		TCCR1B &= ~_BV(CS11); // turn buzzer off!
  		PORTC &= ~_BV(LED1);
	} else {
      		effect_ctrl.beeping |= 0x01;
      		TCCR1B |= _BV(CS11); // turn buzzer on!
  		PORTC |= _BV(LED1);
    		}
  }
  
}

/**********************interrupt for buttons 1 and 3**********************/

void button_press_13 (void)
 {
  // allow interrupts while we're doing this
  sei();
  if (! (PIND & _BV(BUTTON1))) {
    // button1 is pressed
    if (! (last_buttonstate & 0x1)) { // was not pressed before
      PCMSK2 &= ~_BV(PCINT21);        //its better to disable the interrupt here to avoid resets
      delayms(50);                    // debounce
      PCMSK2 |= _BV(PCINT21);         //enable interrupt again
      if (PIND & _BV(BUTTON1))        // filter out bounces
	return;
      tick();                         // make a noise
      // check if we will snag this button press for snoozing
      if (ctrl_struct.alarming) {
	// turn on snooze
	led_off();
	setsnooze();
	return;
      }
      else if (ctrl_struct.countdown_alarming) {
	// turn on snooze
	ctrl_struct.countdown_alarming = FALSE;
	led_off();
	return;
      }
      last_buttonstate |= 0x1;
      just_pressed |= 0x1;
    }
  } else {
    last_buttonstate &= ~0x1;
  }

  if (! (PIND & _BV(BUTTON3))) {
    // button3 is pressed
    if (! (last_buttonstate & 0x4)) { // was not pressed before
      PCMSK2 &= ~_BV(PCINT20);        //its better to disable the interrupt here to avoid resets
      delayms(50);                    // debounce
      PCMSK2 |= _BV(PCINT20);         //enable interrupt again
      if (PIND & _BV(BUTTON3))        // filter out bounces
	return;
      buttonholdcounter = 2;          // see if we're press-and-holding
      while (buttonholdcounter) {
	if (PIND & _BV(BUTTON3)) {        // released
	  	tick();                         // make a noise
	  	last_buttonstate &= ~0x4;
	  	// check if we will snag this button press for snoozing
	  	if (ctrl_struct.alarming) {
	    	// turn on snooze
			led_off();
	   		setsnooze();
	   		return;
	  	}
      		else if (ctrl_struct.countdown_alarming) {
			// turn on snooze
			ctrl_struct.countdown_alarming = FALSE;
			led_off();
			return;
		}
	  	just_pressed |= 0x4;
	  	return;
		}
      	}
      last_buttonstate |= 0x4;
      pressed |= 0x4;                 // held down
    }
  } else {
    pressed = 0;                      // button released
    last_buttonstate &= ~0x4;
  }
}

/*************************Interrupt for button #2****************************/

void button_press_2(void)
 {
  if (! (PINB & _BV(BUTTON2))) {
    // button2 is pressed
    
    if (! (last_buttonstate & 0x2)) { // was not pressed before
      PCMSK0 &= ~_BV(PCINT0);        //its better to disable the interrupt here to avoid resets
      delayms(50);                    // debounce
      PCMSK0 |= _BV(PCINT0);         //enable interrupt again
      if (PINB & _BV(BUTTON2))        // filter out bounces
	{
	return;
	}
      tick();                         // make a noise
      // check if we will snag this button press for snoozing
      if (ctrl_struct.alarming) {
		setsnooze(); 	// turn on snooze
		led_off();
		return;
      }
      else if (ctrl_struct.countdown_alarming) {
		// turn on snooze
		ctrl_struct.countdown_alarming = FALSE;
		led_off();
		return;
	}
      last_buttonstate |= 0x2;
      just_pressed |= 0x2;
    }
  } else {
    last_buttonstate &= ~0x2;
  }
}

/***********************Interrupt for alarm switch**********************/

void alarm_switch(void)
 {
  uint8_t x = ALARM_PIN & _BV(ALARM);
  sei();
  delayms(50); // wait for debouncing
  if (x != (ALARM_PIN & _BV(ALARM)))
    return;
  setalarmstate();
}

/***************Comparator interrupt for detect low-power-mode**********/

void set_standby(void)
 {
  if (ACSR & _BV(ACO)) {
    if (!ctrl_struct.sleepmode) {
      VFDSWITCH_PORT |= _BV(VFDSWITCH); // turn off display
      VFDCLK_PORT &= ~_BV(VFDCLK) & ~_BV(VFDDATA); // no power to vfdchip
      BOOST_PORT &= ~_BV(BOOST); // pull boost fet low
      SPCR  &= ~_BV(SPE); // turn off spi
      //Lets write data to EEPROM
      if (ctrl_struct.restored) {
	eeprom_write_byte((uint8_t *)EE_YEAR, time.y);
	eeprom_write_byte((uint8_t *)EE_MONTH, time.mo);
	eeprom_write_byte((uint8_t *)EE_DAY, time.d);
        eeprom_write_byte((uint8_t *)EE_HOUR, time.h);
	eeprom_write_byte((uint8_t *)EE_MIN, time.m);
	eeprom_write_byte((uint8_t *)EE_SEC, time.s);
	eeprom_write_byte((uint8_t *)EE_ZONE, time.z);
      }
      TCCR0B = 0; // no boost
      ctrl_struct.volume = 0; // low power buzzer
      PCICR = 0;  // ignore buttons

      app_start();
    }
  } else {
    if (ctrl_struct.sleepmode) {
      if (ctrl_struct.restored) {
	eeprom_write_byte((uint8_t *)EE_YEAR, time.y);
	eeprom_write_byte((uint8_t *)EE_MONTH, time.mo);
	eeprom_write_byte((uint8_t *)EE_DAY, time.d);
        eeprom_write_byte((uint8_t *)EE_HOUR, time.h);
	eeprom_write_byte((uint8_t *)EE_MIN, time.m);
	eeprom_write_byte((uint8_t *)EE_SEC, time.s);
	eeprom_write_byte((uint8_t *)EE_ZONE, time.z);
      }
      app_start();
    }
  }
} 

/**************Calculate date, time and other stuff once a second*************/

 void time_calc(void)
 {

  static uint8_t time_divider;

 //call ambilight_pwm 4 times a second
  if (effect_ctrl.light == FALSE)
	{
 	ambilight_pwm_calc();
	}

  time_divider++;
  if (time_divider < 4)
	{
	return;
	}

  //from here with 1 Hz
  time_divider = 0;
  CLKPR = _BV(CLKPCE);  //MEME
  CLKPR = 0;

  time.s++;             // one second has gone by
  overblend = 200;	// new overblending every second;

  // a minute!
  if (time.s >= 60) {
    time.s = 0;
    time.m++;

    // Reset all floating effects so that they will come once a minute
    effect_ctrl.date = TRUE;
    effect_ctrl.message_timer = TRUE;
    effect_ctrl.time = TRUE;

    // decrease countdown timer
    if (countdown && ctrl_struct.countdown_is_set)
	countdown--;
  }

  // an hour...
  if (time.m >= 60) {
    time.m = 0;
    time.h++; 
    calc_timezone();

    //we should store values in eeprom now (its no problem - its possible to write to eeprom 100.000 times)
    eeprom_write_byte((uint8_t *)EE_YEAR, time.y);
    eeprom_write_byte((uint8_t *)EE_MONTH, time.mo);
    eeprom_write_byte((uint8_t *)EE_DAY, time.d);
    eeprom_write_byte((uint8_t *)EE_HOUR, time.h);
    eeprom_write_byte((uint8_t *)EE_MIN, time.m);
    eeprom_write_byte((uint8_t *)EE_SEC, time.s);
    eeprom_write_byte((uint8_t *)EE_ZONE, time.z);
  }

  // a day....
  if (time.h >= 24) {
    time.h = 0;
    time.d++;

    //sync pwm to prevent mismatch between color and clock
    recreate_pwm_sequence();

    //if time.z is true, its switched to wintertime. we have to set it to false again next day. for details see util.c
    if (time.z == TRUE)
	time.z = FALSE;

  }

  // a full month!
  // we check the leapyear and date to verify when its time to roll over months

  if ((time.d > 31) ||
      ((time.d == 31) && ((time.mo == 4)||(time.mo == 6)||(time.mo == 9)||(time.mo == 11))) ||
      ((time.d == 30) && (time.mo == 2)) ||
      ((time.d == 29) && (time.mo == 2) && !leapyear(2000+time.y))) {
    time.d = 1;
    time.mo++;
  }
  
  // HAPPY NEW YEAR!
  if (time.mo >= 13) {
    time.y++;
    time.mo = 1;
  }
  
  // If we're in low power mode we should get out now since the display is off
  if (ctrl_struct.sleepmode)
    return;
   
  // update clock display
  if (displaymode == SHOW_TIME) 
	control_display();

  if ((ctrl_struct.alarm_on && (alarm_h == time.h) && (alarm_m == time.m) && (time.s == 0)))  {
    	ctrl_struct.alarming = TRUE;
    	snoozetimer = 0;
	}

  else if (ctrl_struct.countdown_is_set && countdown == 0)  {
	ctrl_struct.countdown_is_set = FALSE;
	ctrl_struct.countdown_alarming = TRUE;
  	}

  if (timeoutcounter)
    timeoutcounter--;
  if (buttonholdcounter)
    buttonholdcounter--;
  if (snoozetimer) {
    snoozetimer--;
    if (snoozetimer % 2) 
      display[0] |= 0x1;
    else
      display[0] &= ~0x1;
  }
}
