/****************************STUFF***********************************/

// switch to battery mode

void gotosleep(void) {
  
  ctrl_struct.sleepmode = 1;
  VFDSWITCH_PORT |= _BV(VFDSWITCH); // turn off display
  SPCR  &= ~_BV(SPE); // turn off spi
  VFDCLK_PORT &= ~_BV(VFDCLK) & ~_BV(VFDDATA); // no power to vfdchip
  BOOST_PORT &= ~_BV(BOOST); // pull boost fet low
  TCCR0B = 0; // no boost
  ctrl_struct.volume = 0; // low power buzzer
  PCICR = 0;  // ignore buttons

  // turn beeper off
  PORTB &= ~_BV(SPK1) & ~_BV(SPK2); 
  
  //turn off LEDs
  PORTC &= ~_BV(LED1);
  PORTC &= ~_BV(LED2);
  PORTC &= ~_BV(LED3);
  PORTC &= ~_BV(LED4);

  // turn off pullups
  PORTD &= ~_BV(BUTTON1) & ~_BV(BUTTON3);
  PORTB &= ~_BV(BUTTON2);
  DDRD &= ~_BV(BUTTON1) & ~_BV(BUTTON3);
  DDRB &= ~_BV(BUTTON2);
  ALARM_PORT &= ~_BV(ALARM);
  ALARM_DDR &= ~_BV(ALARM);
  

  // reduce the clock speed
  CLKPR = _BV(CLKPCE);
  CLKPR = _BV(CLKPS3);
  
  //  PPR |= _BV(PRUSART0) | _BV(PRADC) | _BV(PRSPI) | _BV(PRTIM1) | _BV(PRTIM0) | _BV(PRTWI);
  PORTC &= ~_BV(LED1);  // sleep signal
  SMCR |= _BV(SM1) | _BV(SM0) | _BV(SE); // sleep mode
  asm("sleep"); 
  CLKPR = _BV(CLKPCE);
  CLKPR = 0;
}

// This will calculate leapyears, give it the year
// and it will return 1 (true) or 0 (false)

uint8_t leapyear(uint16_t y) {
  return ( (!(y % 4) && (y % 100)) || !(y % 400));
}

//Calculate summer/wintertime

void calc_timezone()
{
int dotw = calc_dotw();

if (time.mo == 3 && time.d > 24 && dotw == 0 && time.h == 2)
	{
	time.h++;
	recreate_pwm_sequence(); //recreate pwm colours
	}

else if (time.mo == 10 && time.d > 24 && dotw == 0 && time.h == 3 && time.z == FALSE)
	{
	time.h--;
	time.z = TRUE; //if we dont do this, switch to wintertime will end in an endless loop, because we go one hour back...its not needed in summertime switch, because summertime is one hour forward.
	recreate_pwm_sequence(); //recreate pwm colours
	}
}

// Calculate day of the week

uint8_t calc_dotw()
{
   uint16_t year;
   uint8_t month;
    
   month = time.mo;
   year = 2000 + time.y;
   if (time.mo < 3)  {
	month += 12;
	year -= 1;
   }
return (time.d + (2 * month) + (6 * (month+1)/10) + year + (year/4) - (year/100) + (year/400) + 1) % 7;
}

//Small delay function

void delayms(uint16_t ms) {
  sei();
  milliseconds = 0;
  while (milliseconds < ms);
}

// When the alarm is going off, pressing a button turns on snooze mode
// this sets the snoozetimer off in MAXSNOOZE seconds - which turns on
// the alarm again

void setsnooze(void) {
  snoozetimer = MAXSNOOZE;
  display_str(SNOOZE_MESSAGE, FALSE);
  displaymode = SHOW_SNOOZE;
  delayms(1000);
  displaymode = SHOW_TIME;
}

// we reset the watchdog timer 
void kickthedog(void) {
  wdt_reset();
}

/**************************** ALARM  *****************************/

// This turns on/off the alarm when the switch has been
// set. It also displays the alarm time

void setalarmstate(void) {
  if (ALARM_PIN & _BV(ALARM)) { 
    	// Don't display the alarm/beep if we already have
    	if  (!ctrl_struct.alarm_on) {
   	   	// alarm on!
 	     	ctrl_struct.alarm_on = TRUE;
 	     	// reset snoozing
	      	snoozetimer = 0;
    	 	 // show the status on the VFD tube
  	    	display_str(LANG_ALERT, FALSE);
 	     	// its not actually SHOW_SNOOZE but just anything but SHOW_TIME
 	     	displaymode = SHOW_SNOOZE;
 	     	delayms(1000);
	      	// show the current alarm time set
 	     	display_alarm(alarm_h, alarm_m);
 	     	delayms(1000);
 	     	// after a second, go back to clock mode
 	     	displaymode = SHOW_TIME;
    	}
  } else {
    	if (ctrl_struct.alarm_on) {
     		 // turn off the alarm
     		 ctrl_struct.alarm_on = FALSE;
      		snoozetimer = 0;
      		if (ctrl_struct.alarming) {
			// if the alarm is going off, we should turn it off
			// and quiet the speaker
			ctrl_struct.alarming = FALSE;
			TCCR1B &= ~_BV(CS11); // turn it off!
			PORTB |= _BV(SPK1) | _BV(SPK2);
			PORTC &= ~_BV(LED1); 
      		} 
    	}
  }
}

/****************************SPEAKER *****************************/

// This makes the speaker tick, it doesnt use PWM
// instead it just flicks the piezo

void tick(void) {
  TCCR1A = 0;
  TCCR1B = 0;

  // Send a pulse thru both pins, alternating
  SPK_PORT |= _BV(SPK1);
  SPK_PORT &= ~_BV(SPK2);
  delayms(10);
  SPK_PORT |= _BV(SPK2);
  SPK_PORT &= ~_BV(SPK1);
  delayms(10);
  // turn them both off
  SPK_PORT &= ~_BV(SPK1) & ~_BV(SPK2);

  TCCR1A = _BV(COM1A1) | _BV(COM1B1) | _BV(COM1B0) | _BV(WGM11);
  TCCR1B = _BV(WGM13) | _BV(WGM12);
}

// We can play short beeps!
void beep(uint16_t freq, uint8_t times) {
  // set the PWM output to match the desired frequency
  ICR1 = (F_CPU/8)/freq;
  // we want 50% duty cycle square wave
  OCR1A = OCR1B = ICR1/2;
   
  while (times--) {
    TCCR1B |= _BV(CS11); // turn it on!
    // beeps are 200ms long on
    _delay_ms(200);
    TCCR1B &= ~_BV(CS11); // turn it off!
    PORTB &= ~_BV(SPK1) & ~_BV(SPK2);
    // beeps are 200ms long off
    _delay_ms(200);
  }
  // turn speaker off
  PORTB &= ~_BV(SPK1) & ~_BV(SPK2);
}
/****************************  LED   *****************************/

//Disables the alarm LED

void led_off(void)
	{
	PORTC &= ~_BV(LED1); 
	} 
